unit UnionJack;

interface

uses
  QGraphics, Types;

procedure DrawUnionJack(Canvas: TCanvas; ARect: TRect);

implementation

function RGB(R, G, B: Byte): TColor;
begin
  Result := TColor(R or (G shl 8) or (B shl 16));
end;

var
  Theta, Theta2: Double;
  SinTheta, SinTheta2: Double;

procedure DrawUnionJack(Canvas: TCanvas; ARect: TRect);
var
  L, T, R, B, H, W,
  ThirdHeight, TwoThirdsHeight,
  VertOfs, HorzOfs, HorzOfs2: Integer;
begin
  //Set up brief access variables
  L := ARect.Left;
  T := ARect.Top;
  B := ARect.Bottom;
  R := ARect.Right;
  H := B - T;
  W := R - L;
  ThirdHeight := T + H div 3;
  TwoThirdsHeight := T + H * 2 div 3;
  with Canvas do
  begin
    Brush.Color := clBlack;
    //Blue background
    Brush.Color := RGB(0, 51, 102);
    FillRect(Rect(L, T, R, B));

    //Border for the crosses of St. Andrew and St. Patrick
    Pen.Color := clWhite;
    Brush.Color := Pen.Color;
    //Get horizontal offset of border
    HorzOfs := Round(H / (10 * SinTheta));
    //Get vertical offset of border
    VertOfs := Round(H / (10 * SinTheta2));
    //Draw top left to bottom right diagonal border
    Polygon([
      Point(L, T), Point(L + HorzOfs, T), Point(R, B - VertOfs),
      Point(R, B), Point(R - HorzOfs, B), Point(L, T + VertOfs), Point(L, T)]);
    //Draw bottom left to top right diagonal border
    Polygon([
      Point(L, B), Point(L + HorzOfs, B), Point(R, T + VertOfs),
      Point(R, T), Point(R - HorzOfs, T), Point(L, B - VertOfs), Point(L, B)]);
      
    //Red cross of St Patrick
    Pen.Color := RGB(204, 0, 51);
    Brush.Color := Pen.Color;
    //Get vertical size of strip
    VertOfs := Round(H / (15 * SinTheta2));
    //Get centre-most vertex of strip
    HorzOfs := W div 3;
    //Get other non-border vertex of strip for top-left and bottom-right
    HorzOfs2 := Round((H / 3 - VertOfs) * W/H);
    //Draw top left diagonal strip
    Polygon([Point(L, T), Point(L + HorzOfs, ThirdHeight),
      Point(L + HorzOfs2, ThirdHeight),
      Point(L, T + VertOfs), Point(L, T)]);
    //Draw bottom right diagonal strip
    Polygon([Point(R, B), Point(R - HorzOfs, TwoThirdsHeight),
      Point(R - HorzOfs2, TwoThirdsHeight),
      Point(R, B - VertOfs), Point(R, B)]);
    //Get other non-border vertex of strip for bottom-left and top-right
    HorzOfs2 := HorzOfs + Round(H / (15 * SinTheta));
    //Draw bottom left diagonal
    Polygon([Point(L, B), Point(L + HorzOfs2 - HorzOfs, B),
      Point(L + HorzOfs2, TwoThirdsHeight),
      Point(L + HorzOfs, TwoThirdsHeight), Point(L, B)]);
    //Draw top right diagonal
    Polygon([Point(R, T), Point(R - HorzOfs, ThirdHeight),
      Point(R - HorzOfs2, ThirdHeight),
      Point(R - HorzOfs2 + HorzOfs, T), Point(R, T)]);
    //White cross of St Andrew is already there due to the border being the same colour (white)

    //Border for the red cross of St. George
    //White vertical strip
    Brush.Color := clWhite;
    FillRect(Rect(L + W * 5 div 12, T, L + W * 7 div 12, B));
    //White horizontal strip
    FillRect(Rect(L, T + H div 3, R, T + H * 2 div 3));
    //The cross of St. George
    //Red vertical strip
    Brush.Color := RGB(204, 0, 51);
    FillRect(Rect(L + W * 9 div 20, T, L + W * 11 div 20, T + H));
    //Red horizontal strip
    FillRect(Rect(L, T + H * 2 div 5, R, T + H * 3 div 5));
  end
end;

initialization
  //Get one diagonal angle, where width:height = 2:1
  Theta := ArcTan(1/2);
  SinTheta := Sin(Theta);
  //Get other bisection angle
  Theta2 := ArcTan(2);
  SinTheta2:= Sin(Theta2);
end.
